/*
Firmware for a clock using 7-segment-displays made from resistors and
thermochromic paint.
(C) 2010 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
	    
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
			    
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "io.h"
#include "7seg.h"
#include <avr/io.h>
#include "util/delay.h"
#include <stdio.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include "stdout.h"

volatile char secs=0;
volatile char nextMin=1;

//Timer1 interrupt. Triggers every second.
ISR(TIMER1_CAPT_vect) {
	secs++;
	if (secs>=60) {
		secs=0;
		nextMin=1;
	}
}

//Increase the four digits given. The four digits are seen as a time, e.g. so
//rollover at 60 secs works. The time is 12h-based, not 24h.
void increaseSegs(char *digs) {
	digs[0]++;
	if (digs[0]==10) {
		digs[0]=0;
		digs[1]++;
		if (digs[1]==6) {
			digs[1]=0;
			digs[2]++;
			if (digs[2]==10) {
				digs[2]=0;
				digs[3]=1;
			}
			if (digs[2]==3 && digs[3]==1) {
				digs[2]=1;
				digs[3]=10;
			}
		}
	}
}


int main(void) {
	int x;
	unsigned long val=0;
	unsigned long prevVal=0;
	unsigned long nextVal=0;
	char digits[4]={9, 5, 1, 1}; //12 o'clock blinker :)
	char digitsNext[4]={0,0,0,0};
	ioInit();
	val=charsToShiftVal(digits);

	//Configure timer1 to generate an interrupt once a second.
	TCCR1A=0;
	TCCR1B=0x18+4; //roll over on ICR1, /256
	ICR1=31250; //1 int / sec
	TIMSK=(1<<3); //int on ICR1
	sei();

	while(1) {
		//Wait for rf stuff. Timer routine handles the rtc-ish stuff. Do 
		//something else when the display needs refreshing or the correct rf value
		//has been received.

		//The received RF signal is very simple: off for 620uS and on for 1300
		//implies an 0, off for 1300uS and on for 620 implies an 1.
		//The routine will detect when 0xBn987 is sent, with n being the hour of the
		//day.
		unsigned long v=0;
		do {
			v>>=1;
			while(rfGetVal()) ;
			_delay_us(160);
			if (!rfGetVal()) {
				_delay_us(800);
				if (!rfGetVal()) v|=0x80000;
				_delay_us(400);
			} else {
				v=0;
			}
		} while ((v&0xF0FFF)!=0xB0987 && secs!=0 && secs!=30);

		if ((v&0xF0FFF)==0xB0987) {
			//Set the time to what we received.
			unsigned char no=(v>>12)&0xf;
			secs=0;
			digits[0]=0; digits[1]=0;
			if (no>=10) {
				digits[3]=1;
				digits[2]=no-10;
			} else {
				digits[3]=10;
				digits[2]=no;
			}
		} else if (nextMin) {
			//A minute has passed. Increase digits.
			nextMin=0;
			increaseSegs(digits);
		}

		/*
		Heating happens in 3 steps. First, every segment that is on gets heated for 400ms.
		On the whole minute, every segment is heated for 700mS. If we're 30secs or
		more in this menute, however, only the segments that will be on in the next
		minute too are heated the extra 700mS. Finally, any segments that haven't
		been heated before this mess began get an extra 500mS of heating time.
		*/

		val=charsToShiftVal(digits);
		for (x=0; x<4; x++) digitsNext[x]=digits[x];
		increaseSegs(digitsNext);
		nextVal=charsToShiftVal(digitsNext);
		shiftOutStaggered(val, 400); //heat, standards
		if (secs>=30) val&=nextVal;
		shiftOutStaggered(val, 700); //heat only thingies used in next round
		shiftOutStaggered(val&(~prevVal),500); //first heat extra
		prevVal=val;
	}
}
